diff -Nur c3000_pre/linux/drivers/char/keyboard.c c3000_work/linux/drivers/char/keyboard.c
--- c3000_pre/linux/drivers/char/keyboard.c	2004-12-16 22:55:50.000000000 +0900
+++ c3000_work/linux/drivers/char/keyboard.c	2004-12-19 13:14:53.000000000 +0900
@@ -21,6 +21,10 @@
  *
  * 27-05-97: Added support for the Magic SysRq Key (Martin Mares)
  * 30-07-98: Dead keys redone, aeb@cwi.nl.
+ *
+ * 04-04-1998: Added keyboard autorepeat support (some keyboards don't
+ *   autorepeat, and some keyboard changers interfere with keyboard
+ *   autorepeat settings).     - Russell King (rmk@arm.linux.org.uk)
  */
 
 #include <linux/config.h>
@@ -30,6 +34,7 @@
 #include <linux/tty_flip.h>
 #include <linux/mm.h>
 #include <linux/string.h>
+#include <linux/timer.h>
 #include <linux/random.h>
 #include <linux/init.h>
 
@@ -62,14 +67,29 @@
 #define KBD_DEFLOCK 0
 #endif
 
+/*
+ * Default autorepeat settings.
+ *  DEFAULT_REPEAT_TIMEOUT is the timeout from the keypress to the first repeat
+ *  DEFAULT_REPEAT_INTERVAL is the timeout between successive repeats
+ */
+#define DEFAULT_REPEAT_TIMEOUT	HZ*300/1000
+#define DEFAULT_REPEAT_INTERVAL	HZ*30/1000
+
 void (*kbd_ledfunc)(unsigned int led);
 EXPORT_SYMBOL(handle_scancode);
 EXPORT_SYMBOL(kbd_ledfunc);
 
 extern void ctrl_alt_del(void);
 
+DECLARE_WAIT_QUEUE_HEAD(keypress_wait);
 struct console;
 
+int keyboard_wait_for_keypress(struct console *co)
+{
+	sleep_on(&keypress_wait);
+	return 0;
+}
+
 /*
  * global state includes the following, and various static variables
  * in this module: prev_scancode, shift_state, diacr, npadch, dead_key_next.
@@ -91,11 +111,15 @@
 static int npadch = -1;			/* -1 or number assembled on pad */
 static unsigned char diacr;
 static char rep;			/* flag telling character repeat */
+static int kbd_repeatkeycode= -1;
+static int kbd_repeattimeout = DEFAULT_REPEAT_TIMEOUT;
+static int kbd_repeatinterval= DEFAULT_REPEAT_INTERVAL;
 struct kbd_struct kbd_table[MAX_NR_CONSOLES];
 static struct tty_struct **ttytab;
 static struct kbd_struct * kbd = kbd_table;
 static struct tty_struct * tty;
 
+static void kbd_processkeycode(unsigned char scancode, char up_flag, int autorepeat);
 void compute_shiftstate(void);
 
 typedef void (*k_hand)(unsigned char value, char up_flag);
@@ -162,7 +186,8 @@
  * string, and in both cases we might assume that it is
  * in utf-8 already.
  */
-void to_utf8(ushort c) {
+void to_utf8(ushort c)
+{
     if (c < 0x80)
 	put_queue(c);			/*  0*******  */
     else if (c < 0x800) {
@@ -181,17 +206,23 @@
  * Translation of escaped scancodes to keycodes.
  * This is now user-settable (for machines were it makes sense).
  */
-
 int setkeycode(unsigned int scancode, unsigned int keycode)
 {
-    return kbd_setkeycode(scancode, keycode);
+	return kbd_setkeycode(scancode, keycode);
 }
 
 int getkeycode(unsigned int scancode)
 {
-    return kbd_getkeycode(scancode);
+	return kbd_getkeycode(scancode);
 }
 
+static void key_callback(unsigned long nr);
+
+static struct timer_list key_autorepeat_timer =
+{
+	function: key_callback
+};
+
 void handle_scancode(unsigned char scancode, int down)
 {
 	unsigned char keycode;
@@ -199,6 +230,7 @@
 	char raw_mode;
 
 	pm_access(pm_kbd);
+
 	add_keyboard_randomness(scancode | up_flag);
 
 	tty = ttytab? ttytab[fg_console]: NULL;
@@ -233,12 +265,35 @@
 	 * return the keycode if in MEDIUMRAW mode.
 	 */
 
+	kbd_processkeycode(keycode, up_flag, 0);
+
+out:
+	do_poke_blanked_console = 1;
+	schedule_console_callback();
+}
+
+static void
+kbd_processkeycode(unsigned char keycode, char up_flag, int autorepeat)
+{
+	char raw_mode = (kbd->kbdmode == VC_RAW);
+
 	if (up_flag) {
 		rep = 0;
 		if(!test_and_clear_bit(keycode, key_down))
 		    up_flag = kbd_unexpected_up(keycode);
-	} else
+	} else {
 		rep = test_and_set_bit(keycode, key_down);
+		/* If the keyboard autorepeated for us, ignore it.
+		 * We do our own autorepeat processing.
+		 */
+		if (rep && !autorepeat)
+			return;
+	}
+
+	if (kbd_repeatkeycode == keycode || !up_flag || raw_mode) {
+		kbd_repeatkeycode = -1;
+		del_timer(&key_autorepeat_timer);
+	}
 
 #ifdef CONFIG_MAGIC_SYSRQ		/* Handle the SysRq Hack */
 	if (keycode == SYSRQ_KEY) {
@@ -252,6 +307,23 @@
 	}
 #endif
 
+	/*
+	 * Calculate the next time when we have to do some autorepeat
+	 * processing.  Note that we do not do autorepeat processing
+	 * while in raw mode but we do do autorepeat processing in
+	 * medium raw mode.
+	 */
+	if (!up_flag && !raw_mode) {
+		kbd_repeatkeycode = keycode;
+		if (vc_kbd_mode(kbd, VC_REPEAT)) {
+			if (rep)
+				key_autorepeat_timer.expires = jiffies + kbd_repeatinterval;
+			else
+				key_autorepeat_timer.expires = jiffies + kbd_repeattimeout;
+			add_timer(&key_autorepeat_timer);
+		}
+	}
+
 	if (kbd->kbdmode == VC_MEDIUMRAW) {
 		/* soon keycodes will require more than one byte */
 		put_queue(keycode + up_flag);
@@ -320,37 +392,39 @@
 #endif
 		}
 	}
+	rep = 0;
 out:
-	do_poke_blanked_console = 1;
-	schedule_console_callback();
 }
 
-#ifdef CONFIG_FORWARD_KEYBOARD
-extern int forward_chars;
-
-void put_queue(int ch)
+/*
+ * This clears the key down arrays when the keyboard is reset.  On
+ * keyboard reset, this must be called before any keycodes are
+ * received.
+ */
+void kbd_reset_kdown(void)
 {
-	if (forward_chars == fg_console+1){
-		kbd_forward_char (ch);
-	} else {
-		if (tty) {
-			tty_insert_flip_char(tty, ch, 0);
-			con_schedule_flip(tty);
-		}
-	}
+	int i;
+
+	for (i = 0; i < NR_SHIFT; i++)
+		k_down[i] = 0;
+	for (i = 0; i < SIZE(key_down); i++)
+		key_down[i] = 0;
+	shift_state = 0;
 }
-#else
+
+
 void put_queue(int ch)
 {
+	wake_up(&keypress_wait);
 	if (tty) {
 		tty_insert_flip_char(tty, ch, 0);
 		con_schedule_flip(tty);
 	}
 }
-#endif
 
 static void puts_queue(char *cp)
 {
+	wake_up(&keypress_wait);
 	if (!tty)
 		return;
 
@@ -524,7 +598,7 @@
 {
 }
 
-static void do_null()
+static void do_null(void)
 {
 	compute_shiftstate();
 }
@@ -639,8 +713,8 @@
 
 static void do_pad(unsigned char value, char up_flag)
 {
-	static const char *pad_chars = "0123456789+-*/\015,.?()";
-	static const char *app_map = "pqrstuvwxylSRQMnnmPQ";
+	static const char *pad_chars = "0123456789+-*/\015,.?()#";
+	static const char *app_map = "pqrstuvwxylSRQMnnmPQS";
 
 	if (up_flag)
 		return;		/* no action, if this is a key release */
@@ -741,9 +815,10 @@
 	}
 }
 
-/* called after returning from RAW mode or when changing consoles -
-   recompute k_down[] and shift_state from key_down[] */
-/* maybe called when keymap is undefined, so that shiftkey release is seen */
+/* Called after returning from RAW mode or when changing consoles -
+ * recompute k_down[] and shift_state from key_down[]
+ * Maybe called when keymap is undefined so that shift key release is seen
+ */
 void compute_shiftstate(void)
 {
 	int i, j, k, sym, val;
@@ -822,19 +897,22 @@
 }
 
 /*
- * The leds display either (i) the status of NumLock, CapsLock, ScrollLock,
- * or (ii) whatever pattern of lights people want to show using KDSETLED,
- * or (iii) specified bits of specified words in kernel memory.
+ * The leds display either
+ * (i)   the status of NumLock, CapsLock, ScrollLock, or
+ * (ii)  whatever pattern of lights people want to show using KDSETLED, or
+ * (iii) specified bits of specified words in kernel memory.
  */
 
 static unsigned char ledstate = 0xff; /* undefined */
 static unsigned char ledioctl;
 
-unsigned char getledstate(void) {
+unsigned char getledstate(void)
+{
     return ledstate;
 }
 
-void setledstate(struct kbd_struct *kbd, unsigned int led) {
+void setledstate(struct kbd_struct *kbd, unsigned int led)
+{
     if (!(led & ~7)) {
 	ledioctl = led;
 	kbd->ledmode = LED_SHOW_IOCTL;
@@ -850,7 +928,8 @@
 } ledptrs[3];
 
 void register_leds(int console, unsigned int led,
-		   unsigned int *addr, unsigned int mask) {
+		   unsigned int *addr, unsigned int mask)
+{
     struct kbd_struct *kbd = kbd_table + console;
     if (led < 3) {
 	ledptrs[led].addr = addr;
@@ -861,7 +940,8 @@
 	kbd->ledmode = LED_SHOW_FLAGS;
 }
 
-static inline unsigned char getleds(void){
+static inline unsigned char getleds(void)
+{
     struct kbd_struct *kbd = kbd_table + fg_console;
     unsigned char leds;
 
@@ -908,6 +988,19 @@
 {
 	unsigned char leds = getleds();
 
+	if (rep && kbd_repeatkeycode != -1) {
+		tty = ttytab? ttytab[fg_console]: NULL;
+		kbd = kbd_table + fg_console;
+
+		/* This prevents the kbd_key routine from being called
+		 * twice, once by this BH, and once by the interrupt
+		 * routine.
+		 */
+		kbd_disable_irq();
+		kbd_processkeycode(kbd_repeatkeycode, 0, 1);
+		kbd_enable_irq();
+	}
+
 	if (leds != ledstate) {
 		ledstate = leds;
 		kbd_leds(leds);
@@ -918,6 +1011,12 @@
 EXPORT_SYMBOL(keyboard_tasklet);
 DECLARE_TASKLET_DISABLED(keyboard_tasklet, kbd_bh, 0);
 
+static void key_callback(unsigned long nr)
+{
+	rep = 1;
+	tasklet_schedule(&keyboard_tasklet);
+}
+
 typedef void (pm_kbd_func) (void);
 
 pm_callback pm_kbd_request_override = NULL;
@@ -967,7 +1066,7 @@
 
 	tasklet_enable(&keyboard_tasklet);
 	tasklet_schedule(&keyboard_tasklet);
-	
+
 	pm_kbd = pm_register(PM_SYS_DEV, PM_SYS_KBC, pm_kbd_request_override);
 
 	return 0;
diff -Nur c3000_pre/linux/drivers/char/spitz_logkey.c c3000_work/linux/drivers/char/spitz_logkey.c
--- c3000_pre/linux/drivers/char/spitz_logkey.c	2004-11-04 14:10:12.000000000 +0900
+++ c3000_work/linux/drivers/char/spitz_logkey.c	2004-12-19 12:55:33.000000000 +0900
@@ -221,8 +221,8 @@
 int (*mach_kbdrate) (struct kbd_repeat *);
 
 
-#define DEFAULT_KEYB_REP_DELAY  (HZ*2)
-#define DEFAULT_KEYB_REP_RATE   (HZ)
+#define DEFAULT_KEYB_REP_DELAY  (HZ)
+#define DEFAULT_KEYB_REP_RATE   (HZ/4)
 
 static void sharppda_kbd_rep( unsigned long ignore );
 static unsigned char rep_scancode = 0;
